/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.auditlog;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.trifork.sosigw.Factory;
import com.trifork.sosigw.api.Intercom;
import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.config.RuntimeConfigItem;
import com.trifork.sosigw.config.RuntimeConfigState;
import com.trifork.sosigw.console.datamodel.AuditLogEntry;
import com.trifork.sosigw.console.datamodel.AuditLogQuery;


public class AuditLogSearchTest extends SqlTests {

    @BeforeClass
    public static void initMySqlTests() {
    	
    	initSqlTests();
    }

	
    @Before
    public void setup() {
        RuntimeConfig.setRuntimeConfigState(new RuntimeConfigState(1L, new RuntimeConfigItem[] {
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCINTERVAL, new String[] { "100" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCURL, new String[] { "jdbc:hsqldb:mem:test" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCUSERNAME, new String[] { "sa" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCPASSWORD, new String[] { "" })
        }));

        Intercom ic = mock(Intercom.class);
        Factory.setIntercom(ic);
        
        when(ic.getUID()).thenReturn("uid");
    }
    
    @After
    public void shutdown() {
        AuditLogGlobalPostgresql.stop();
    }
    
    @Test
    public void testSearch() throws Exception {
    	// Given
        AuditLogGlobalPostgresql.start();
        
        AuditLogGlobalPostgresql globalPostgresql = new AuditLogGlobalPostgresql();

        AuditLogQuery queryAll = new AuditLogQuery();
        AuditLogQuery query1111111118 = new AuditLogQuery();
        query1111111118.setCpr("1111111118");
        
        int countAll = globalPostgresql.search(queryAll).size();
        int countMatching1111111118 = globalPostgresql.search(query1111111118).size();
        

        // When
        AuditLogPostgresql al = new AuditLogPostgresql();
        al.log(new AuditLogItem("test", "1111111118", System.currentTimeMillis(), "http://localhost", "urn:action", "127.0.0.1"));
        al.log(new AuditLogItem("test", "3333333333", System.currentTimeMillis(), "http://localhost", "urn:action", "127.0.0.1"));
        Thread.sleep(200);

        // Then
        assertEquals("Forventede to ekstra auditlog entries", countAll + 2, globalPostgresql.search(queryAll).size());
        
        List<AuditLogEntry> list = globalPostgresql.search(query1111111118);
        assertEquals("Forventede 1 ekstra auditlog entry på cprnummeret", countMatching1111111118 + 1, list.size());
        for (AuditLogEntry auditLogEntry : list) {
            assertEquals("1111111118", auditLogEntry.getCpr());
        }
    }
}
